@@ -41,56 +41,60 @@ function HeuristicBlocker(pbStorage) {
 }
 
 HeuristicBlocker.prototype = {
-  /**
-   * Adds Cookie blocking for all more specific domains than the blocked origin
-   * - if they're on the cb list
-   *
-   * @param {String} origin Origin to check
-   */
-  setupSubdomainsForCookieblock: function(origin) {
-    var cbl = this.storage.getBadgerStorageObject("cookieblock_list");
-    for (var domain in cbl.getItemClones()) {
-      if (origin == window.getBaseDomain(domain)) {
-        this.storage.setupHeuristicAction(domain, constants.COOKIEBLOCK);
-      }
-    }
-    // iterate through all elements of cookie block list
-    // if element has basedomain add it to action_map
-    // or update it's action with cookieblock
-    origin = null;
-    return false;
-  },
 
   /**
-   * Decide if to blacklist and add blacklist filters
-   * @param {String} baseDomain The base domain (etld+1) to blacklist
+   * Blocklists an FQDN/origin:
+   *
+   * - Blocks or cookieblocks an FQDN.
+   * - Blocks or cookieblocks its base domain.
+   * - Cookieblocks any yellowlisted subdomains that share the base domain with the FQDN.
+   *
+   * @param {String} base The base domain (etld+1) to blocklist
    * @param {String} fqdn The FQDN
    */
-  blacklistOrigin: function(baseDomain, fqdn) {
-    var cbl = this.storage.getBadgerStorageObject("cookieblock_list");
+  blocklistOrigin: function (base, fqdn) {
+    let self = this,
+      ylistStorage = self.storage.getBadgerStorageObject("cookieblock_list");
 
-    // Setup Cookieblock or block for base domain and fqdn
-    if (cbl.hasItem(baseDomain)) {
-      this.storage.setupHeuristicAction(baseDomain, constants.COOKIEBLOCK);
+    // cookieblock or block the base domain
+    if (ylistStorage.hasItem(base)) {
+      self.storage.setupHeuristicAction(base, constants.COOKIEBLOCK);
     } else {
-      this.storage.setupHeuristicAction(baseDomain, constants.BLOCK);
+      self.storage.setupHeuristicAction(base, constants.BLOCK);
     }
 
-    // Check if a parent domain of the fqdn is on the cookie block list
-    var set = false;
-    var thisStorage = this.storage;
-    _.each(utils.explodeSubdomains(fqdn, true), function(domain) {
-      if (cbl.hasItem(domain)) {
-        thisStorage.setupHeuristicAction(fqdn, constants.COOKIEBLOCK);
+    // cookieblock or block the fqdn
+    //
+    // cookieblock if a "parent" domain of the fqdn is on the yellowlist
+    //
+    // ignore base domains when exploding to work around PSL TLDs:
+    // still want to cookieblock somedomain.googleapis.com with only
+    // googleapis.com (and not somedomain.googleapis.com itself) on the ylist
+    let set = false,
+      subdomains = utils.explodeSubdomains(fqdn, true);
+    for (let i = 0; i < subdomains.length; i++) {
+      if (ylistStorage.hasItem(subdomains[i])) {
         set = true;
+        break;
+      }
+    }
+    if (set) {
+      self.storage.setupHeuristicAction(fqdn, constants.COOKIEBLOCK);
+    } else {
+      self.storage.setupHeuristicAction(fqdn, constants.BLOCK);
+    }
+
+    // cookieblock any yellowlisted subdomains with the same base domain
+    //
+    // for example, when google.com is blocked,
+    // books.google.com should be cookieblocked
+    let base_with_dot = '.' + base;
+    ylistStorage.keys().forEach(domain => {
+      if (base != domain && domain.endsWith(base_with_dot)) {
+        self.storage.setupHeuristicAction(domain, constants.COOKIEBLOCK);
       }
     });
-    // if no parent domains are on the cookie block list then block fqdn
-    if (!set) {
-      this.storage.setupHeuristicAction(fqdn, constants.BLOCK);
-    }
 
-    this.setupSubdomainsForCookieblock(baseDomain);
   },
 
   /**
@@ -146,12 +150,10 @@ HeuristicBlocker.prototype = {
    * Wraps _recordPrevalence for use outside of webRequest listeners.
    *
    * @param {String} tracker_fqdn The fully qualified domain name of the tracker
-   * @param {String} page_origin The base domain of the page
-   *   where the tracker was detected.
-   * @param {Boolean} skip_dnt_check Skip DNT policy checking if flag is true.
-   *
+   * @param {String} tracker_origin Base domain of the third party tracker
+   * @param {String} page_origin Base domain of page where tracking occurred
    */
-  updateTrackerPrevalence: function(tracker_fqdn, page_origin, skip_dnt_check) {
+  updateTrackerPrevalence: function (tracker_fqdn, tracker_origin, page_origin) {
     // abort if we already made a decision for this fqdn
     let action = this.storage.getAction(tracker_fqdn);
     if (action != constants.NO_TRACKING && action != constants.ALLOW) {
@@ -160,9 +162,8 @@ HeuristicBlocker.prototype = {
 
     this._recordPrevalence(
       tracker_fqdn,
-      window.getBaseDomain(tracker_fqdn),
-      page_origin,
-      skip_dnt_check
+      tracker_origin,
+      page_origin
     );
   },
 
@@ -177,11 +178,9 @@ HeuristicBlocker.prototype = {
    *
    * @param {String} tracker_fqdn The FQDN of the third party tracker
    * @param {String} tracker_origin Base domain of the third party tracker
-   * @param {String} page_origin The origin of the page where the third party
-   *   tracker was loaded.
-   * @param {Boolean} skip_dnt_check Skip DNT policy checking if flag is true.
+   * @param {String} page_origin Base domain of page where tracking occurred
    */
-  _recordPrevalence: function (tracker_fqdn, tracker_origin, page_origin, skip_dnt_check) {
+  _recordPrevalence: function (tracker_fqdn, tracker_origin, page_origin) {
     var snitchMap = this.storage.getBadgerStorageObject('snitch_map');
     var firstParties = [];
     if (snitchMap.hasItem(tracker_origin)) {
@@ -198,15 +197,6 @@ HeuristicBlocker.prototype = {
       return; // We already know about the presence of this tracker on the given domain
     }
 
-    // Check this just-seen-tracking-on-this-site,
-    // not-yet-blocked domain for DNT policy.
-    // We check heuristically-blocked domains in webrequest.js.
-    if (!skip_dnt_check) {
-      setTimeout(function () {
-        badger.checkForDNTPolicy(tracker_fqdn);
-      }, 0);
-    }
-
     // record that we've seen this tracker on this domain (in snitch map)
     firstParties.push(page_origin);
     snitchMap.setItem(tracker_origin, firstParties);
@@ -214,8 +204,6 @@ HeuristicBlocker.prototype = {
     // ALLOW indicates this is a tracker still below TRACKING_THRESHOLD
     // (vs. NO_TRACKING for resources we haven't seen perform tracking yet).
     // see https://github.com/EFForg/privacybadger/pull/1145#discussion_r96676710
-    // TODO missing tests: removing below lines/messing up parameters
-    // should break integration tests, but currently does not
     this.storage.setupHeuristicAction(tracker_fqdn, constants.ALLOW);
     this.storage.setupHeuristicAction(tracker_origin, constants.ALLOW);
 
@@ -224,8 +212,8 @@ HeuristicBlocker.prototype = {
 
     // block the origin if it has been seen on multiple first party domains
     if (httpRequestPrevalence >= constants.TRACKING_THRESHOLD) {
-      log('blacklisting origin', tracker_fqdn);
-      this.blacklistOrigin(tracker_origin, tracker_fqdn);
+      log('blocklisting origin', tracker_fqdn);
+      this.blocklistOrigin(tracker_origin, tracker_fqdn);
     }
   }
 };
@@ -502,7 +490,8 @@ function hasCookieTracking(details, origin) {
       }
 
       // ignore CloudFlare
-      if (name == "__cfduid") {
+      // https://support.cloudflare.com/hc/en-us/articles/200170156-Understanding-the-Cloudflare-Cookies
+      if (name == "__cfduid" || name == "__cf_bm") {
         continue;
       }
 
