@@ -15,34 +15,26 @@
  * along with Privacy Badger.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-var utils = require("utils");
-var constants = require("constants");
+require.scopes.migrations = (function () {
 
-require.scopes.migrations = (function() {
+let utils = require("utils");
+let constants = require("constants");
+
+let noop = function () {};
+
+let exports = {};
 
-var exports = {};
 exports.Migrations= {
-  changePrivacySettings: function() {
-    if (!chrome.extension.inIncognitoContext && chrome.privacy) {
-      console.log('changing privacy settings');
-      if (chrome.privacy.services && chrome.privacy.services.alternateErrorPagesEnabled) {
-        chrome.privacy.services.alternateErrorPagesEnabled.set({'value': false, 'scope': 'regular'});
-      }
-      if (chrome.privacy.websites && chrome.privacy.websites.hyperlinkAuditingEnabled) {
-        chrome.privacy.websites.hyperlinkAuditingEnabled.set({'value': false, 'scope': 'regular'});
-      }
-    }
-  },
-
-  migrateAbpToStorage: function () {},
+  changePrivacySettings: noop,
+  migrateAbpToStorage: noop,
 
   migrateBlockedSubdomainsToCookieblock: function(badger) {
     setTimeout(function() {
       console.log('MIGRATING BLOCKED SUBDOMAINS THAT ARE ON COOKIE BLOCK LIST');
-      var cbl = badger.storage.getBadgerStorageObject('cookieblock_list');
-      _.each(badger.storage.getAllDomainsByPresumedAction(constants.BLOCK), function(fqdn) {
-        _.each(utils.explodeSubdomains(fqdn, true), function(domain) {
-          if (cbl.hasItem(domain)) {
+      let ylist = badger.storage.getBadgerStorageObject('cookieblock_list');
+      badger.storage.getAllDomainsByPresumedAction(constants.BLOCK).forEach(fqdn => {
+        utils.explodeSubdomains(fqdn, true).forEach(domain => {
+          if (ylist.hasItem(domain)) {
             console.log('moving', fqdn, 'from block to cookie block');
             badger.storage.setupHeuristicAction(fqdn, constants.COOKIEBLOCK);
           }
@@ -51,7 +43,7 @@ exports.Migrations= {
     }, 1000 * 30);
   },
 
-  migrateLegacyFirefoxData: function() { },
+  migrateLegacyFirefoxData: noop,
 
   migrateDntRecheckTimes: function(badger) {
     var action_map = badger.storage.getBadgerStorageObject('action_map');
@@ -171,7 +163,7 @@ exports.Migrations= {
       if (sites && sites.length) {
         if (sites.length >= constants.TRACKING_THRESHOLD) {
           // tracking domain over threshold, set it to cookieblock or block
-          badger.heuristicBlocking.blacklistOrigin(base_domain, domain);
+          badger.heuristicBlocking.blocklistOrigin(base_domain, domain);
           continue;
 
         } else {
@@ -209,7 +201,7 @@ exports.Migrations= {
     // reblock all blocked domains to trigger yellowlist logic
     for (let i = 0; i < blocked.length; i++) {
       let domain = blocked[i];
-      badger.heuristicBlocking.blacklistOrigin(
+      badger.heuristicBlocking.blocklistOrigin(
         window.getBaseDomain(domain), domain);
     }
   },
@@ -228,25 +220,7 @@ exports.Migrations= {
     }
   },
 
-  resetWebRTCIPHandlingPolicy: function (badger) {
-    console.log("Resetting webRTCIPHandlingPolicy ...");
-
-    if (!badger.webRTCAvailable) {
-      return;
-    }
-
-    const cpn = chrome.privacy.network;
-
-    cpn.webRTCIPHandlingPolicy.get({}, function (result) {
-      if (!result.levelOfControl.endsWith('_by_this_extension')) {
-        return;
-      }
-
-      if (result.value == 'default_public_interface_only') {
-        cpn.webRTCIPHandlingPolicy.clear({});
-      }
-    });
-  },
+  resetWebRTCIPHandlingPolicy: noop,
 
   enableShowNonTrackingDomains: function (badger) {
     console.log("Enabling showNonTrackingDomains for some users");
@@ -345,6 +319,35 @@ exports.Migrations= {
     badger.storage.forget("consensu.org");
   },
 
+  resetWebRTCIPHandlingPolicy2: function (badger) {
+    if (!badger.webRTCAvailable) {
+      return;
+    }
+
+    const cpn = chrome.privacy.network;
+
+    cpn.webRTCIPHandlingPolicy.get({}, function (result) {
+      if (!result.levelOfControl.endsWith('_by_this_extension')) {
+        return;
+      }
+
+      // migrate default (disabled) setting for old Badger versions
+      // from Mode 3 to Mode 1
+      if (result.value == 'default_public_interface_only') {
+        console.log("Resetting webRTCIPHandlingPolicy ...");
+        cpn.webRTCIPHandlingPolicy.clear({});
+
+      // migrate enabled setting for more recent Badger versions
+      // from Mode 4 to Mode 3
+      } else if (result.value == 'disable_non_proxied_udp') {
+        console.log("Updating WebRTC IP leak protection setting ...");
+        cpn.webRTCIPHandlingPolicy.set({
+          value: 'default_public_interface_only'
+        });
+      }
+    });
+  }
+
 };
 
 
