@@ -1,10 +1,10 @@
-require.scopes.multiDomainFP = (function() {
+require.scopes.multiDomainFP = (function () {
 
 /**
  * 2d array of related domains (etld+1), all domains owned by the same entity go into
  * an array, this is later transformed for efficient lookups.
  */
-var multiDomainFirstPartiesArray = [
+let multiDomainFirstPartiesArray = [
   ["1800contacts.com", "800contacts.com"],
   ["37signals.com", "basecamp.com", "basecamphq.com", "highrisehq.com"],
   ["9gag.com", "9cache.com"],
@@ -155,26 +155,147 @@ var multiDomainFirstPartiesArray = [
 
     "muscache.com",
   ],
+  [
+    "airfranceklm.com",
+
+    "airfrance.cg",
+    "airfrance.ci",
+    "airfrance.com.cn",
+    "airfrance.com.do",
+    "airfrance.com.gh",
+    "airfrance.dz",
+    "airfrance.id",
+    "airfrance.in",
+    "airfrance.my",
+    "airfrance.ng",
+    "airfrance.pa",
+    "airfrance.tn",
+    "airfrance.vn",
+
+    "klm.ae",
+    "klm.at",
+    "klm.aw",
+    "klm.be",
+    "klm.bg",
+    "klm.by",
+    "klm.bz",
+    "klm.ca",
+    "klm.ch",
+    "klm.cl",
+    "klm.co.ao",
+    "klm.co.cr",
+    "klm.co.id",
+    "klm.co.il",
+    "klm.co.in",
+    "klm.co.jp",
+    "klm.co.ke",
+    "klm.co.kr",
+    "klm.com",
+    "klm.com.ar",
+    "klm.com.au",
+    "klm.com.bh",
+    "klm.com.br",
+    "klm.com.co",
+    "klm.com.cy",
+    "klm.com.ec",
+    "klm.com.eg",
+    "klm.com.gh",
+    "klm.com.hk",
+    "klm.com.mt",
+    "klm.com.mx",
+    "klm.com.my",
+    "klm.com.na",
+    "klm.com.ng",
+    "klm.com.pa",
+    "klm.com.pe",
+    "klm.com.ph",
+    "klm.com.py",
+    "klm.com.qa",
+    "klm.com.tr",
+    "klm.com.tw",
+    "klm.com.uy",
+    "klm.co.nz",
+    "klm.co.th",
+    "klm.co.tz",
+    "klm.co.ug",
+    "klm.co.uk",
+    "klm.co.za",
+    "klm.co.zm",
+    "klm.cw",
+    "klm.cz",
+    "klm.de",
+    "klm.dk",
+    "klm.do",
+    "klm.es",
+    "klm.fi",
+    "klm.fr",
+    "klm.ge",
+    "klm.gr",
+    "klm.hr",
+    "klm.hu",
+    "klm.ie",
+    "klm.it",
+    "klm.kz",
+    "klm.lk",
+    "klm.lt",
+    "klm.lu",
+    "klm.lv",
+    "klm.mu",
+    "klm.mw",
+    "klm.nc",
+    "klm.nl",
+    "klm.no",
+    "klm.pl",
+    "klm.pt",
+    "klm.ro",
+    "klm.ru",
+    "klm.se",
+    "klm.sg",
+    "klm.sk",
+    "klm.sr",
+    "klm.sx",
+    "klm.ua",
+    "klm.us",
+
+    "static-af.com",
+    "static-kl.com",
+  ],
   [
     "alibaba.com",
 
     "1688.com",
     "95095.com",
     "9game.cn",
+    "aliapp.org",
+    "alibabacloud.co.in",
+    "alibabacloud.com",
+    "alibabacloud.com.au",
+    "alibabacloud.com.hk",
+    "alibabacloud.com.my",
+    "alibabacloud.com.sg",
+    "alibabacloud.com.tw",
+    "alibabacorp.com",
     "alibabagroup.com",
     "alibaba-inc.com",
     "alicdn.com",
     "alicdn.net",
+    "alicloud.com",
     "aliexpress.com",
     "aliexpress.ru",
+    "alifanyi.com",
     "aligames.com",
     "alihealth.cn",
     "alihive.com",
     "aliimg.com",
     "alimama.com",
+    "alimei.com",
+    "aliplus.com",
     "alitrip.com",
     "alitrip.hk",
     "aliyun.com",
+    "aliyuncs.com",
+    "aliyun-iot-share.com",
+    "amap.com",
     "cainiao.com",
     "cainiao.com.cn",
     "cibntv.net",
@@ -203,12 +324,21 @@ var multiDomainFirstPartiesArray = [
     "mmstat.com",
     "mobmore.com",
     "paike.com",
+    "phpwind.com",
+    "phpwind.net",
     "puata.info",
     "soku.com",
+    "sparenode.com",
+    "supet.com",
     "tanx.com",
     "taobao.com",
     "taopiaopiao.com",
     "tbcdn.cn",
+    "tburl.in",
+    "teambitionapis.com",
+    "teambition.com",
+    "teambition.net",
+    "tianchi-global.com",
     "tmail.com",
     "tmall.com",
     "tmall.hk",
@@ -240,6 +370,7 @@ var multiDomainFirstPartiesArray = [
     "alipay.cn",
     "alipay-eco.com",
     "alipay.hk",
+    "alipayobjects.com",
     "alipayplus.com",
     "ant-biz.com",
     "ant-financial.com",
@@ -413,6 +544,7 @@ var multiDomainFirstPartiesArray = [
   ["apple.com", "icloud.com", "icloud.com.cn", "cdn-apple.com"],
   ["applefcu.org", "applefcuonline.org"],
   ["archive.org", "openlibrary.org"],
+  ["asos.com", "asosservices.com"],
   [
     "atlassian.com",
 
@@ -429,7 +561,15 @@ var multiDomainFirstPartiesArray = [
 
     "atl-paas.net",
   ],
-  ["att.com", "att.tv"],
+  [
+    "att.com",
+
+    "att.tv",
+    "atttvnow.com",
+    "attwatchtv.com",
+    "directv.com",
+    "directvnow.com",
+  ],
   [
     "autodesk.com",
 
@@ -475,7 +615,16 @@ var multiDomainFirstPartiesArray = [
   ],
   ["avito.ru", "avito.st"],
   ["avon.com", "youravon.com"],
-  ["baidu.com", "bdimg.com", "bdstatic.com"],
+  [
+    "baidu.com",
+
+    "hao123.com",
+    "tieba.com",
+
+    "baidustatic.com",
+    "bdimg.com",
+    "bdstatic.com",
+  ],
   ["balsamiq.com", "balsamiq.cloud"],
   ["bancomer.com", "bancomer.com.mx", "bbvanet.com.mx"],
   ["bankofamerica.com", "bofa.com", "mbna.com", "usecfo.com"],
@@ -493,10 +642,41 @@ var multiDomainFirstPartiesArray = [
   ["belkin.com", "seedonk.com"],
   ["bhphotovideo.com", "bandh.com", "bhphoto.com", "bnh.com"],
   ["bilibili.com", "acgvideo.com", "biliapi.net", "biliapi.com", "biligame.com", "hdslb.com"],
+  ["binance.com", "binance.cloud", "binance.vision", "bnbstatic.com"],
   ["blizzard.com", "battle.net", "worldofwarcraft.com"],
   ["bloomberg.com", "bbthat.com", "bwbx.io"],
+  ["boardgamearena.com", "boardgamearena.net"],
   ["booking.com", "bstatic.com"],
   ["box.com", "boxcdn.net"],
+  [
+    "bustle.company",
+
+    "bustle.com",
+    "elitedaily.com",
+    "inputmag.com",
+    "inverse.com",
+    "mic.com",
+    "nylon.com",
+    "romper.com",
+    "thezoereport.com",
+  ],
+  [
+    "canalplus.com",
+
+    "canal.fr",
+    "canalplay.com",
+    "canalplus-bo.net",
+    "canal-plus.com",
+    "canalplus.fr",
+    "canalplusinternational.com",
+    "canal-plus.net",
+    "canal-plus.pro",
+    "canalplus.pro",
+    "canalpro.fr",
+    "clique.tv",
+    "cstar.fr",
+    "mycanal.fr",
+  ],
   ["capitalone.com", "capitalone360.com"],
   [
     "cbs.com",
@@ -506,6 +686,7 @@ var multiDomainFirstPartiesArray = [
     "cbsimg.net",
     "cbsinteractive.com",
     "cbsistatic.com",
+    "cbslocal.com",
     "cbsnews.com",
     "cbssports.com",
     "cbsstatic.com",
@@ -552,8 +733,20 @@ var multiDomainFirstPartiesArray = [
   ],
   ["century21.com", "21online.com"],
   ["chart.io", "chartio.com"],
+  ["chaturbate.com", "highwebmedia.com"],
   ["cms.gov", "medicare.gov", "mymedicare.gov"],
+  ["codepen.io", "cdpn.io"],
   ["concur.com", "concursolutions.com"],
+  [
+    "cornell.edu",
+
+    "birdsna.org",
+    "birdsoftheworld.com",
+    "birdsoftheworld.org",
+    "ebird.org",
+    "hbw.com",
+    "macaulaylibrary.org",
+  ],
   ["cox.com", "cox.net"],
   ["cricketwireless.com", "aiowireless.com"],
   ["ctrip.com", "c-ctrip.com", "trip.com"],
@@ -691,7 +884,9 @@ var multiDomainFirstPartiesArray = [
     "directferries.xyz",
   ],
   ["discountbank.co.il", "telebank.co.il"],
+  ["discord.com", "discordapp.com", "discordapp.net"],
   ["discover.com", "discovercard.com"],
+  ["disqus.com", "disquscdn.com"],
   [
     "dpgmediagroup.com",
 
@@ -725,7 +920,15 @@ var multiDomainFirstPartiesArray = [
   ],
   ["dropbox.com", "dropboxstatic.com", "dropboxusercontent.com", "getdropbox.com"],
   ["d.rip", "kickstarter.com"],
-  ["ea.com", "origin.com", "play4free.com", "tiberiumalliance.com"],
+  [
+    "ea.com",
+
+    "bioware.com",
+    "masseffect.com",
+    "origin.com",
+    "play4free.com",
+    "tiberiumalliance.com",
+  ],
   [
     "ebay.com",
     "ebayinc.com",
@@ -837,6 +1040,7 @@ var multiDomainFirstPartiesArray = [
     "alamo-np.fr",
     "alamo-np.ie",
   ],
+  ["epicgames.com", "unrealengine.com"],
   [
     "expedia.com",
 
@@ -915,6 +1119,36 @@ var multiDomainFirstPartiesArray = [
     "ebookers.no",
 
     "mrjet.dk",
+
+    "vrbo.com",
+
+    "abritel.fr",
+    "aluguetemporada.com.br",
+    "fewo-direkt.de",
+    "homeaway.at",
+    "homeaway.ca",
+    "homeaway.com",
+    "homeaway.com.au",
+    "homeaway.com.mx",
+    "homeaway.co.nz",
+    "homeaway.co.uk",
+    "homeaway.dk",
+    "homeaway.es",
+    "homeaway.fi",
+    "homeaway.gr",
+    "homeaway.it",
+    "homeaway.nl",
+    "homeaway.no",
+    "homeaway.pl",
+    "homeaway.pt",
+    "homeaway.se",
+    "homelidays.com",
+    "homelidays.es",
+    "homelidays.fr",
+    "homelidays.it",
+    "ownersdirect.co.uk",
+    "stayz.com.au",
+    "vacationrentals.com",
   ],
   ["express-scripts.com", "medcohealth.com"],
   [
@@ -992,6 +1226,36 @@ var multiDomainFirstPartiesArray = [
   ["fastmail.com", "fastmailusercontent.com"],
   ["firefox.com", "firefoxusercontent.com", "mozilla.org"],
   ["foxnews.com", "foxbusiness.com", "fncstatic.com"],
+  [
+    "futureplc.com",
+
+    "creativebloq.com",
+    "cyclingnews.com",
+    "digitalcameraworld.com",
+    "gamesradar.com",
+    "gizmodo.co.uk",
+    "guitarworld.com",
+    "kotaku.co.uk",
+    "laptopmag.com",
+    "lifehacker.co.uk",
+    "livescience.com",
+    "loudersound.com",
+    "musicradar.com",
+    "pcgamer.com",
+    "space.com",
+    "t3.com",
+    "techradar.com",
+    "tomsguide.com",
+    "tomshardware.com",
+    "toptenreviews.com",
+    "whathifi.com",
+
+    "futurecdn.net",
+    "future-fie-assets.co.uk",
+    "future-fie.co.uk",
+    "future.net.uk",
+  ],
+  ["gamestar.de", "gamepro.de", "cgames.de"],
   [
     "gap.com",
 
@@ -1014,6 +1278,24 @@ var multiDomainFirstPartiesArray = [
 
     "assets-gap.com",
   ],
+  [
+    "gedispa.it",
+
+    "capital.it",
+    "deejay.it",
+    "gelocal.it",
+    "ilsecoloxix.it",
+    "kataweb.it",
+    "lastampa.it",
+    "lescienze.it",
+    "limesonline.com",
+    "m2o.it",
+    "mymovies.it",
+    "repubblica.it",
+
+    "gedidigital.it",
+    "repstatic.it",
+  ],
   [
     "gettyimages.com",
 
@@ -1030,8 +1312,25 @@ var multiDomainFirstPartiesArray = [
     "thinkstockphotos.ca",
   ],
   ["github.com", "githubapp.com"],
-  ["gizmodo.com", "kinja-img.com", "kinja-static.com", "deadspin.com", "lifehacker.com",
-    "technoratimedia.com", "kinja.com", "jalopnik.com", "jezebel.com"],
+  ["gitlab.com", "gitlab-static.net"],
+  [
+    "gizmodo.com",
+
+    "avclub.com",
+    "deadspin.com",
+    "jalopnik.com",
+    "jezebel.com",
+    "kinja.com",
+    "kinja-img.com",
+    "kinja-static.com",
+    "kotaku.com",
+    "lifehacker.com",
+    "technoratimedia.com",
+    "theinventory.com",
+    "theonion.com",
+    "theroot.com",
+    "thetakeout.com",
+  ],
   [
     "glassdoor.com",
 
@@ -1259,6 +1558,7 @@ var multiDomainFirstPartiesArray = [
     "nest.com",
     "nestpowerproject.withgoogle.com",
   ],
+  ["www.gov.uk", "cabinet-office.gov.uk", "publishing.service.gov.uk"],
   [
     "gray.tv",
 
@@ -1351,42 +1651,97 @@ var multiDomainFirstPartiesArray = [
   [
     "hearst.com",
 
+    "25ans.jp",
+    "autoweek.com",
+    "bazaar.com",
     "beaumontenterprise.com",
+    "bestproducts.com",
+    "bicycling.com",
+    "caranddriver.com",
     "chron.com",
+    "cosmopolitan.com",
+    "countryliving.com",
+    "crfashionbook.com",
     "ctnews.com",
     "ctpost.com",
     "dariennewsonline.com",
+    "delish.com",
+    "drozthegoodlife.com",
+    "elle.com",
+    "elledecor.com",
+    "esquire.com",
     "expressnews.com",
     "fairfieldcitizenonline.com",
     "foothillstrader.com",
+    "fujingaho.jp",
     "gametimect.com",
+    "gearpatrol.com",
+    "ghsealapplication.com",
+    "goodhouse.com",
+    "goodhousekeeping.com",
     "greenwichtime.com",
+    "harpersbazaar.com",
+    "housebeautiful.com",
     "houstonchronicle.com",
     "lmtonline.com",
+    "marieclaire.com",
+    "menshealth.com",
     "michigansthumb.com",
     "middletownpress.com",
     "mrt.com",
     "myjournalcourier.com",
+    "mylo.id",
     "myplainview.com",
     "mysanantonio.com",
     "newcanaannewsonline.com",
     "newmilfordspectrum.com",
     "newstimes.com",
     "nhregister.com",
+    "oprahmag.com",
     "ourmidland.com",
+    "popularmechanics.com",
+    "prevention.com",
+    "redbookmag.com",
     "registercitizen.com",
+    "roadandtrack.com",
+    "rodalesorganiclife.com",
+    "runnersworld.com",
     "seattlepi.com",
+    "seventeen.com",
     "sfchronicle.com",
     "sfgate.com",
+    "shondaland.com",
     "stamfordadvocate.com",
+    "s-w-e-e-t.com",
     "thehour.com",
     "theintelligencer.com",
+    "thepioneerwoman.com",
+    "thepioneerwomancooks.com",
     "thetelegraph.com",
     "timesunion.com",
+    "todays-rewards.com",
+    "townandcountrymag.com",
+    "veranda.com",
+    "wearesweet.co",
     "westport-news.com",
+    "womansday.com",
+    "womenshealthmag.com",
     "yourconroenews.com",
 
+    "h-cdn.co",
+    "hdmtech.net",
+    "hdmtools.com",
     "hdnux.com",
+    "hearst3pcc.com",
+    "hearstapps.com",
+    "hearstapps.net",
+    "hearstdigitalstudios.com",
+    "hearstdigitalstudios.net",
+    "hearst.io",
+    "hearstlabs.com",
+    "hearstmags.com",
+    "hearstmobile.com",
+    "hearstnp.com",
   ],
   [
     "houzz.com",
@@ -1431,28 +1786,52 @@ var multiDomainFirstPartiesArray = [
     "thathomesite.com",
   ],
   [
-    "huffpost.com",
+    "huobi.com",
 
-    "huffingtonpost.ca",
-    "huffingtonpost.com",
-    "huffingtonpost.com.au",
-    "huffingtonpost.com.mx",
-    "huffingtonpost.co.uk",
-    "huffingtonpost.co.za",
-    "huffingtonpost.de",
-    "huffingtonpost.es",
-    "huffingtonpost.fr",
-    "huffingtonpost.gr",
-    "huffingtonpost.in",
-    "huffingtonpost.it",
-    "huffingtonpost.jp",
-    "huffingtonpost.kr",
-    "huffpostarabi.com",
-    "huffpostbrasil.com",
-    "huffpostmaghreb.com",
+    "hbfile.net",
+    "hbg.com",
+    "huobiasia.vip",
+    "huobi.br.com",
+    "huobi.me",
   ],
   ["hvfcu.org", "hvfcuonline.org"],
-  ["ign.fr", "geoportail.gouv.fr", "geoportail-urbanisme.gouv.fr", "cartoradio.fr", "ignrando.fr"],
+  [
+    "idealo.de",
+
+    "idealo.at",
+    "idealo.co.uk",
+    "idealo.es",
+    "idealo.fr",
+    "idealo.it",
+    "idealo.com",
+  ],
+  [
+    "ign.fr",
+
+    "cartoradio.fr",
+    "culture.fr",
+    "duministeredelaculture.fr",
+    "gouvernement.fr",
+    "ignrando.fr",
+
+    "ants.gouv.fr",
+    "culture.gouv.fr",
+    "data.gouv.fr",
+    "education.gouv.fr",
+    "etalab.gouv.fr",
+    "geoportail.gouv.fr",
+    "geoportail-urbanisme.gouv.fr",
+    "impots.gouv.fr",
+    "premier-ministre.gouv.fr",
+    "service-civique.gouv.fr",
+    "yvelines.gouv.fr",
+
+    "ac-grenoble.fr",
+    "ac-versailles.fr",
+    "ac-bordeaux.fr",
+    "ac-montpellier.fr",
+    "ac-lille.fr",
+  ],
   [
     "impresa.pt",
 
@@ -1471,7 +1850,25 @@ var multiDomainFirstPartiesArray = [
     "tribunaexpresso.pt",
     "volantesic.pt",
   ],
+  [
+    "immobilienscout24.de",
+    "static-immobilienscout24.de",
+  ],
   ["independent.co.uk", "indy100.com"],
+  [
+    "iu.edu",
+
+    "indiana.edu",
+    "iue.edu",
+    "iufw.edu",
+    "iuk.edu",
+    "iun.edu",
+    "iupuc.edu",
+    "iupui.edu",
+    "iusb.edu",
+    "ius.edu",
+    "myiu.org",
+  ],
   [
     "jd.com",
     "3.cn",
@@ -1486,7 +1883,9 @@ var multiDomainFirstPartiesArray = [
     "jd.id",
     "jd.ru",
     "jdpay.com",
+    "jdwl.com",
     "jdx.com",
+    "jkcsjd.com",
     "joybuy.com",
     "joybuy.es",
     "ocwms.com",
@@ -1498,6 +1897,8 @@ var multiDomainFirstPartiesArray = [
     "yiyaojd.com",
     "yizhitou.com",
   ],
+  ["jpmorganchase.com", "jpmorgan.com", "chase.com"],
+  ["jobware.de", "jobware.com", "jobware.net"],
   [
     "kayak.com",
 
@@ -1608,7 +2009,11 @@ var multiDomainFirstPartiesArray = [
     "koganmobile.co.nz",
     "kogansuper.com.au",
     "kogantravel.com",
+    "mattblatt.com.au",
+    "tandy.com.au",
+    "zazz.com.au",
   ],
+  ["linkedin.com", "licdn.com"],
   ["livejournal.com", "livejournal.net", "lj-toys.com"],
   ["lnk.to", "tix.to", "tck.to", "ticket.to", "linkfire.com", "assetlab.io", "linkfire.co", "lnkfi.re"],
   [
@@ -1650,6 +2055,22 @@ var multiDomainFirstPartiesArray = [
 
     "1gr.cz",
   ],
+  [
+    "mail.ru",
+    "imgsmail.ru",
+
+    "ok.ru",
+    "mycdn.me",
+    "odnoklassniki.ru",
+    "oklive.app",
+    "ok.me",
+    "tamtam.chat",
+    "tt.me",
+
+    "vk.com",
+    "vk.me",
+    "vkontakte.ru",
+  ],
   ["mandtbank.com", "mtb.com"],
   ["mathletics.com", "mathletics.com.au", "mathletics.co.uk"],
   ["mdsol.com", "imedidata.com"],
@@ -1934,7 +2355,6 @@ var multiDomainFirstPartiesArray = [
     "suomiareena.fi",
   ],
   ["my-bookings.org", "my-bookings.cc"],
-  ["mycanal.fr", "canal-plus.com"],
   [
     "myheritage.com",
 
@@ -2155,12 +2575,8 @@ var multiDomainFirstPartiesArray = [
 
     "wnyc.org",
   ],
-  ["nytimes.com", "newyorktimes.com", "nyt.com"],
-  [
-    "nyu.edu",
-
-    "nyupress.org",
-  ],
+  ["nytimes.com", "newyorktimes.com", "thewirecutter.com", "nyt.com"],
+  ["nyu.edu", "nyupress.org"],
   [
     "nvidia.com",
 
@@ -2223,6 +2639,7 @@ var multiDomainFirstPartiesArray = [
     "psyarxiv.com",
     "thesiscommons.org"
   ],
+  ["osu.edu", "osumc.edu", "ohio-state.edu"],
   [
     "ovh.com",
 
@@ -2247,7 +2664,7 @@ var multiDomainFirstPartiesArray = [
 
     "ovh.net",
   ],
-  ["paypal.com", "paypal-search.com"],
+  ["paypal.com", "paypal-search.com", "paypalobjects.com"],
   ["pcworld.com", "staticworld.net", "idg.com", "idg.net", "infoworld.com", "macworld.com", "techhive.com", "idg.tv"],
   [
     "pearson.com",
@@ -2340,6 +2757,44 @@ var multiDomainFirstPartiesArray = [
     "pricerunner.se",
     "pricerunner.uk",
   ],
+  [
+    "prosiebensat1.de",
+    "prosiebensat1.com",
+
+    "atv.at",
+    "atv2.at",
+    "galileo.tv",
+    "kabeleins.at",
+    "kabeleins.ch",
+    "kabeleins.de",
+    "kabeleinsdoku.at",
+    "kabeleinsdoku.ch",
+    "kabeleinsdoku.de",
+    "prosieben.at",
+    "prosieben.ch",
+    "prosieben.de",
+    "prosiebenmaxx.at",
+    "prosiebenmaxx.ch",
+    "prosiebenmaxx.de",
+    "puls24.at",
+    "puls4.com",
+    "puls8.ch",
+    "ran.de",
+    "sat1.at",
+    "sat1.ch",
+    "sat1.de",
+    "sat1gold.at",
+    "sat1gold.ch",
+    "sat1gold.de",
+    "sixx.at",
+    "sixx.ch",
+    "sixx.de",
+    "the-voice-of-germany.at",
+    "the-voice-of-germany.ch",
+    "the-voice-of-germany.de",
+
+    "p7s1.io",
+  ],
   [
     "qantas.com",
 
@@ -2381,15 +2836,48 @@ var multiDomainFirstPartiesArray = [
   ],
   [
     "qq.com",
+
+    "aitangyou.com",
+    "cdntips.com",
     "dnspod.cn",
+    "extqq.com",
+    "gdtimg.com",
     "gtimg.cn",
     "gtimg.com",
+    "idqqimg.com",
+    "imqq.com",
+    "myapp.com",
+    "myqcloud.com",
     "qcloud.com",
+    "qpic.cn",
+    "qqmail.com",
+    "qzone.com",
     "tencent.com",
+    "tenpay.com",
+    "ugdtimg.com",
+    "url.cn",
     "wechat.com",
     "wegame.com",
     "weiyun.com",
   ],
+  [
+    "rai.it",
+
+    "comunitaitalofona.org",
+    "raicinema.it",
+    "raicultura.it",
+    "raimemo.it",
+    "rainews24.it",
+    "rainews.it",
+    "raiplay.it",
+    "raiplayradio.it",
+    "raiplayyoyo.it",
+    "raipubblicita.it",
+    "raisport.it",
+    "raitalia.it",
+    "rai.tv",
+    "raiway.it",
+  ],
   ["railnation.ru", "railnation.de", "rail-nation.com", "railnation.gr", "railnation.us", "trucknation.de", "traviangames.com"],
   ["rakuten.com", "buy.com"],
   [
@@ -2861,12 +3349,107 @@ var multiDomainFirstPartiesArray = [
     "cdntvn.pl",
   ],
   ["tvp.pl", "tvp.info"],
-  ["twitter.com", "twimg.com", "t.co"],
+  ["twitter.com", "twimg.com", "t.co", "periscope.tv", "pscp.tv"],
   ["ua2go.com", "ual.com", "united.com", "unitedwifi.com"],
   ["ubisoft.com", "ubi.com", "anno-union.com", "thesettlers-alliance.com"],
   ["ui.com", "ubnt.com"],
+  [
+    "unitedhealthgroup.com",
+
+    "myprotectwell.com",
+    "protectwellapp.com",
+    "protectwell.org",
+    "uhg.com",
+    "unitedhealthgroup.net",
+    "unitedhealthgroup.org",
+    "weprotectwell.com",
+
+    "healthyourway.com",
+    "healthyourwaynow.com",
+    "joinatyourbest.com",
+    "mypersonalizedsupport.com",
+    "myrenewactive.com",
+    "mywellbeingsolution.com",
+    "optummessenger.co",
+    "personalhealthmessagecenter.com",
+    "phs.com",
+    "pwrfitness.com",
+    "takechargeatwork.com",
+    "wellnesscoachingnow.com",
+
+    "aarpmedicareplans.com",
+    "careimprovementplus.com",
+    "health4me.com",
+    "healthsafe-id.com",
+    "myaarpmedicare.com",
+    "myaarpmedicareplans.com",
+    "myaarprenew.com",
+    "myaarprenewma.com",
+    "myaarprenewmapd.com",
+    "myaarprenewpdp.com",
+    "myaarpsupplementalhealthinsurance.com",
+    "mymedicamedicare.com",
+    "mymedicareaccount.com",
+    "mypcpmedicare.com",
+    "myuhc.com",
+    "myuhcdental.com",
+    "myuhcmedicare.com",
+    "uhc.com",
+    "uhcmedicaresolutions.com",
+    "uhcmycarepath.com",
+    "uhcretiree.com",
+    "uhcservices.com",
+    "yourdentalplan.com",
+
+    "werally.com",
+
+    "liveandworkwell.com",
+    "myoptum.com",
+    "optum.com",
+    "optumrx.com",
+    "ppconline.info",
+    "ppconlineinfo.com",
+    "wellbeing-4life.com",
+  ],
   ["vanguard.com", "investornews.vanguard", "vanguardblog.com", "vanguardcanada.ca", "vanguardinvestor.co.uk", "vgcontent.info"],
-  ["verizon.com", "verizon.net", "verizonwireless.com", "vzw.com"],
+  [
+    "verizonmedia.com",
+
+    "verizon.com",
+    "verizon.net",
+    "verizonwireless.com",
+    "vzw.com",
+
+    "aol.com",
+    "autoblog.com",
+    "engadget.com",
+    "oath.com",
+    "overture.com",
+    "techcrunch.com",
+    "yahoo.com",
+
+    "huffpost.com",
+    "huffingtonpost.ca",
+    "huffingtonpost.com",
+    "huffingtonpost.com.au",
+    "huffingtonpost.com.mx",
+    "huffingtonpost.co.uk",
+    "huffingtonpost.co.za",
+    "huffingtonpost.de",
+    "huffingtonpost.es",
+    "huffingtonpost.fr",
+    "huffingtonpost.gr",
+    "huffingtonpost.in",
+    "huffingtonpost.it",
+    "huffingtonpost.jp",
+    "huffingtonpost.kr",
+    "huffpostarabi.com",
+    "huffpostbrasil.com",
+    "huffpostmaghreb.com",
+
+    "yahooapis.com",
+    "yimg.com",
+  ],
   ["vimeo.com", "vimeocdn.com"],
   [
     "vinted.com",
@@ -2882,7 +3465,6 @@ var multiDomainFirstPartiesArray = [
     "vinted.nl",
     "vinted.pl",
   ],
-  ["vk.com", "vk.me", "vkontakte.ru"],
   ["volvooceanrace.com", "virtualregatta.com"],
   ["vonage.com", "vonagebusiness.com"],
   [
@@ -2898,6 +3480,7 @@ var multiDomainFirstPartiesArray = [
     "sporza.be",
     "stubru.be",
   ],
+  ["wa.gov", "wsdot.com", "wsdot.gov"],
   ["walmart.com", "wal.co", "walmartimages.com", "walmart.ca"],
   [
     "thewaltdisneycompany.com",
@@ -2930,6 +3513,7 @@ var multiDomainFirstPartiesArray = [
     "disneyinteractive.com",
     "disneyinternationalhd.com",
     "disneymoviesanywhere.com",
+    "disneyplus.com",
     "ilm.com",
     "marvel.com",
     "readriordan.com",
@@ -3021,10 +3605,15 @@ var multiDomainFirstPartiesArray = [
     "filmstruck.com",
     "greatbig.com",
     "greatbigstory.com",
+    "hbo.com",
+    "hbogo.com",
+    "hbomax.com",
+    "hbonow.com",
     "heaveninc.com",
     "hlntv.com",
     "iamthenight.com",
     "juniorrydercup.com",
+    "maxgo.com",
     "nba.com",
     "nba.net",
     "ncaa.com",
@@ -3092,12 +3681,32 @@ var multiDomainFirstPartiesArray = [
     "medscapestatic.com",
   ],
   ["weebly.com", "editmysite.com"],
+  [
+    "weibo.com",
+
+    "sina.cn",
+    "sina.com",
+    "sina.com.cn",
+    "sinaedge.com",
+    "sinaimg.cn",
+    "sinaimg.com",
+    "sinaimg.com.cn",
+    "sinajs.cn",
+    "sina.net",
+    "wbimg.cn",
+    "wbimg.com",
+    "wcdn.cn",
+    "weibocdn.com",
+    "weibo.cn",
+    "weibopay.com",
+  ],
   ["wellsfargo.com", "wf.com"],
   ["wetter.com", "tiempo.es", "wettercomassets.com"],
   ["wikipedia.org", "wikimedia.org", "wikimediafoundation.org", "wiktionary.org",
     "wikiquote.org", "wikibooks.org", "wikisource.org", "wikinews.org",
     "wikiversity.org", "mediawiki.org", "wikidata.org", "wikivoyage.org",
     "wmfusercontent.org", "tools.wmflabs.org"],
+  ["wisconsin.gov", "wi.gov"],
   ["wix.com", "wixapps.net", "wixstatic.com", "parastorage.com"],
   ["wordpress.com", "wp.com"],
   [
@@ -3135,7 +3744,7 @@ var multiDomainFirstPartiesArray = [
   ["xda-developers.com", "xda-cdn.com"],
   ["xfinity.com", "comcast.net", "comcast.com"],
   ["xhamster.com", "xhcdn.com"],
-  ["yahoo.com", "yahooapis.com", "yimg.com", "yahoo.co.jp", "overture.com", "flickr.com"],
+  ["yahoo.co.jp", "yimg.jp"],
   [
     "yandex.com",
 
@@ -3171,8 +3780,26 @@ var multiDomainFirstPartiesArray = [
     "ya.ru",
   ],
   ["zendesk.com", "zopim.com"],
+  ["zhaopin.com", "zhaopin.cn"],
   ["zillow.com", "zillowstatic.com", "zillowcloud.com", "zg-api.com"],
-  ["zoho.com", "zoho.eu", "zohocorp.com", "zohocreator.com", "zohopublic.com", "zohostatic.com"],
+  [
+    "zoho.com",
+
+    "zoho.com.au",
+    "zoho.eu",
+
+    "zohositescontent.com",
+    "zohositescontent.com.au",
+    "zohositescontent.eu",
+
+    "manageengine.com",
+
+    "zohocdn.com",
+    "zohocorp.com",
+    "zohocreator.com",
+    "zohopublic.com",
+    "zohostatic.com",
+  ],
   ["zonealarm.com", "zonelabs.com"],
 ];
 
