@@ -17,17 +17,6 @@
 
 window.OPTIONS_INITIALIZED = false;
 
-// TODO hack: disable Tooltipster tooltips on Firefox to avoid unresponsive script warnings
-(function () {
-const matches = navigator.userAgent.match(
-  // from https://gist.github.com/ticky/3909462
-  /(MSIE|(?!Gecko.+)Firefox|(?!AppleWebKit.+Chrome.+)Safari|(?!AppleWebKit.+)Chrome|AppleWebKit(?!.+Chrome|.+Safari)|Gecko(?!.+Firefox))(?: |\/)([\d.apre]+)/
-);
-if (!matches || matches[1] == "Firefox") {
-  $.fn.tooltipster = function () {};
-}
-}());
-
 const TOOLTIP_CONF = {
   maxWidth: 200
 };
@@ -50,8 +39,8 @@ function loadOptions() {
   document.title = i18n.getMessage("options_title");
 
   // Add event listeners
-  $("#whitelistForm").on("submit", addWhitelistDomain);
-  $("#removeWhitelist").on("click", removeWhitelistDomain);
+  $("#allowlist-form").on("submit", addDisabledSite);
+  $("#remove-disabled-site").on("click", removeDisabledSite);
   $("#cloud-upload").on("click", uploadCloud);
   $("#cloud-download").on("click", downloadCloud);
   $('#importTrackerButton').on("click", loadFileChooser);
@@ -84,11 +73,20 @@ function loadOptions() {
   $("#tracking-domains-show-not-yet-blocked").on("change", filterTrackingDomains);
 
   // Add event listeners for origins container.
-  $(function () {
-    $('#blockedResourcesContainer').on('change', 'input:radio', updateOrigin);
-    $('#blockedResourcesContainer').on('click', '.userset .honeybadgerPowered', revertDomainControl);
-    $('#blockedResourcesContainer').on('click', '.removeOrigin', removeOrigin);
+  $('#blockedResourcesContainer').on('change', 'input:radio', function () {
+    let $radio = $(this),
+      $clicker = $radio.parents('.clicker').first(),
+      origin = $clicker.data('origin'),
+      action = $radio.val();
+
+    // update domain slider row tooltip/status indicators
+    updateOrigin(origin, action, true);
+
+    // persist the change
+    saveToggle(origin, action);
   });
+  $('#blockedResourcesContainer').on('click', '.userset .honeybadgerPowered', revertDomainControl);
+  $('#blockedResourcesContainer').on('click', '.removeOrigin', removeOrigin);
 
   // Display jQuery UI elements
   $("#tabs").tabs({
@@ -99,8 +97,8 @@ function loadOptions() {
     }
   });
   $("button").button();
-  $(".addButton").button("option", "icons", {primary: "ui-icon-plus"});
-  $(".removeButton").button("option", "icons", {primary: "ui-icon-minus"});
+  $("#add-disabled-site").button("option", "icons", {primary: "ui-icon-plus"});
+  $("#remove-disabled-site").button("option", "icons", {primary: "ui-icon-minus"});
   $("#cloud-upload").button("option", "icons", {primary: "ui-icon-arrowreturnthick-1-n"});
   $("#cloud-download").button("option", "icons", {primary: "ui-icon-arrowreturnthick-1-s"});
   $(".importButton").button("option", "icons", {primary: "ui-icon-plus"});
@@ -176,7 +174,7 @@ function loadOptions() {
   widgetSelector.on('select2:unselect', updateWidgetReplacementExceptions);
   widgetSelector.on('select2:clear', updateWidgetReplacementExceptions);
 
-  reloadWhitelist();
+  reloadDisabledSites();
   reloadTrackingDomainsTab();
 
   $('html').css('visibility', 'visible');
@@ -253,7 +251,7 @@ function parseUserDataFile(storageMapsList) {
     OPTIONS_DATA.disabledSites = response.disabledSites;
     OPTIONS_DATA.origins = response.origins;
 
-    reloadWhitelist();
+    reloadDisabledSites();
     reloadTrackingDomainsTab();
     // TODO general settings are not updated
 
@@ -286,7 +284,7 @@ function downloadCloud() {
       if (response.success) {
         alert(i18n.getMessage("download_cloud_success"));
         OPTIONS_DATA.disabledSites = response.disabledSites;
-        reloadWhitelist();
+        reloadDisabledSites();
       } else {
         console.error("Cloud sync error:", response.message);
         if (response.message === i18n.getMessage("download_cloud_no_data")) {
@@ -462,22 +460,24 @@ function updateLearnInIncognito() {
   });
 }
 
-function reloadWhitelist() {
-  var sites = OPTIONS_DATA.disabledSites;
-  var sitesList = $('#excludedDomainsBox');
-  // Sort the white listed sites in the same way the blocked sites are
+function reloadDisabledSites() {
+  let sites = OPTIONS_DATA.disabledSites,
+    $select = $('#allowlist-select');
+
+  // sort disabled sites the same way blocked sites are sorted
   sites = htmlUtils.sortDomains(sites);
-  sitesList.html("");
-  for (var i = 0; i < sites.length; i++) {
-    $('<option>').text(sites[i]).appendTo(sitesList);
+
+  $select.empty();
+  for (let i = 0; i < sites.length; i++) {
+    $('<option>').text(sites[i]).appendTo($select);
   }
 }
 
-function addWhitelistDomain(event) {
+function addDisabledSite(event) {
   event.preventDefault();
 
-  var domain = utils.getHostFromDomainInput(
-    document.getElementById("newWhitelistDomain").value.replace(/\s/g, "")
+  let domain = utils.getHostFromDomainInput(
+    document.getElementById("new-disabled-site-input").value.replace(/\s/g, "")
   );
 
   if (!domain) {
@@ -489,16 +489,16 @@ function addWhitelistDomain(event) {
     domain
   }, (response) => {
     OPTIONS_DATA.disabledSites = response.disabledSites;
-    reloadWhitelist();
-    document.getElementById("newWhitelistDomain").value = "";
+    reloadDisabledSites();
+    document.getElementById("new-disabled-site-input").value = "";
   });
 }
 
-function removeWhitelistDomain(event) {
+function removeDisabledSite(event) {
   event.preventDefault();
 
   let domains = [];
-  let $selected = $("#excludedDomainsBox option:selected");
+  let $selected = $("#allowlist-select option:selected");
   for (let i = 0; i < $selected.length; i++) {
     domains.push($selected[i].text);
   }
@@ -508,7 +508,7 @@ function removeWhitelistDomain(event) {
     domains
   }, (response) => {
     OPTIONS_DATA.disabledSites = response.disabledSites;
-    reloadWhitelist();
+    reloadDisabledSites();
   });
 }
 
@@ -522,25 +522,30 @@ function getOriginAction(origin) {
   return OPTIONS_DATA.origins[origin];
 }
 
-function revertDomainControl(e) {
-  var $elm = $(e.target).parent();
-  var origin = $elm.data('origin');
+function revertDomainControl(event) {
+  event.preventDefault();
+
+  let origin = $(event.target).parent().data('origin');
+
   chrome.runtime.sendMessage({
     type: "revertDomainControl",
     origin
   }, (response) => {
+    // update any sliders that changed as a result
+    updateSliders(response.origins);
+    // update cached domain data
     OPTIONS_DATA.origins = response.origins;
-    reloadTrackingDomainsTab();
   });
 }
 
 /**
  * Displays list of all tracking domains along with toggle controls.
  */
-function reloadTrackingDomainsTab() {
+function updateSummary() {
   // Check to see if any tracking domains have been found before continuing.
-  var allTrackingDomains = getOriginsArray(OPTIONS_DATA.origins, null, null, null, true);
-  if (!allTrackingDomains || allTrackingDomains.length === 0) {
+  let allTrackingDomains = getOriginsArray(OPTIONS_DATA.origins, null, null, null, true);
+
+  if (!allTrackingDomains || !allTrackingDomains.length) {
     // leave out number of trackers and slider instructions message if no sliders will be displayed
     $("#options_domain_list_trackers").hide();
     $("#options_domain_list_one_tracker").hide();
@@ -573,10 +578,17 @@ function reloadTrackingDomainsTab() {
     $("#options_domain_list_trackers").html(i18n.getMessage(
       "options_domain_list_trackers", [
         baseDomains.size,
-        "<a target='_blank' title='" + _.escape(i18n.getMessage("what_is_a_tracker")) + "' class='tooltip' href='https://www.eff.org/privacybadger/faq#What-is-a-third-party-tracker'>"
+        "<a target='_blank' title='" + _.escape(i18n.getMessage("what_is_a_tracker")) + "' class='tooltip' href='https://privacybadger.org/#What-is-a-third-party-tracker'>"
       ]
     )).show();
   }
+}
+
+/**
+ * Displays list of all tracking domains along with toggle controls.
+ */
+function reloadTrackingDomainsTab() {
+  updateSummary();
 
   // Get containing HTML for domain list along with toggle legend icons.
   $("#blockedResources")[0].innerHTML = htmlUtils.getTrackerContainerHtml();
@@ -633,69 +645,31 @@ function filterTrackingDomains() {
   }, timeToWait);
 }
 
-/**
- * Registers handlers for tracking domain toggle controls.
- * @param {jQuery} $toggleElement jQuery object for the tracking domain element to be registered.
- */
-// TODO unduplicate this code? since a version of it is also in popup
-function registerToggleHandlers($toggleElement) {
-  var radios = $toggleElement.children('input');
-  var value = $toggleElement.children('input:checked').val();
-
-  var slider = $('<div></div>').slider({
-    min: 0,
-    max: 2,
-    value: value,
-    create: function(/*event, ui*/) {
-      // Set the margin for the handle of the slider we're currently creating,
-      // depending on its blocked/cookieblocked/allowed value (this == .ui-slider)
-      $(this).children('.ui-slider-handle').css('margin-left', -16 * value + 'px');
-    },
-    slide: function(event, ui) {
-      radios.filter('[value=' + ui.value + ']').click();
-    },
-    stop: function(event, ui) {
-      $(ui.handle).css('margin-left', -16 * ui.value + 'px');
-
-      // Save change for origin.
-      var origin = radios.filter('[value=' + ui.value + ']')[0].name;
-      var setting = htmlUtils.getCurrentClass($toggleElement.parents('.clicker'));
-      chrome.runtime.sendMessage({
-        type: "saveOptionsToggle",
-        action: setting,
-        origin: origin
-      }, (response) => {
-        OPTIONS_DATA.origins = response.origins;
-        reloadTrackingDomainsTab();
-      });
-    },
-  }).appendTo($toggleElement);
-
-  radios.on("change", function() {
-    slider.slider('value', radios.filter(':checked').val());
-  });
-}
-
 /**
  * Adds more origins to the blocked resources list on scroll.
  *
 */
 function addOrigins(e) {
-  var domains = e.data;
-  var target = e.target;
-  var totalHeight = target.scrollHeight - target.clientHeight;
-  if ((totalHeight - target.scrollTop) < 400) {
-    var domain = domains.shift();
-    var action = getOriginAction(domain);
+  let domains = e.data;
+  if (!domains.length) {
+    return;
+  }
+
+  let el = e.target;
+  let total_height = el.scrollHeight - el.clientHeight;
+  if ((total_height - el.scrollTop) >= 400) {
+    return;
+  }
+
+  for (let i = 0; (i < 50) && (domains.length > 0); i++) {
+    let domain = domains.shift();
+    let action = getOriginAction(domain);
     if (action) {
       let show_breakage_warning = (
         action == constants.USER_BLOCK &&
         OPTIONS_DATA.cookieblocked.hasOwnProperty(domain)
       );
-      $(target).append(htmlUtils.getOriginHtml(domain, action, show_breakage_warning));
-
-      // register the newly-created toggle switch so that user changes are saved
-      registerToggleHandlers($(target).find("[data-origin='" + domain + "'] .switch-toggle"));
+      $(el).append(htmlUtils.getOriginHtml(domain, action, show_breakage_warning));
     }
   }
 
@@ -712,21 +686,21 @@ function showTrackingDomains(domains) {
   domains = htmlUtils.sortDomains(domains);
 
   // Create HTML for the initial list of tracking domains.
-  var trackingDetails = '';
-  for (var i = 0; (i < 50) && (domains.length > 0); i++) {
-    var domain = domains.shift();
-    var action = getOriginAction(domain);
+  let out = '';
+  for (let i = 0; (i < 50) && (domains.length > 0); i++) {
+    let domain = domains.shift();
+    let action = getOriginAction(domain);
     if (action) {
       let show_breakage_warning = (
         action == constants.USER_BLOCK &&
         OPTIONS_DATA.cookieblocked.hasOwnProperty(domain)
       );
-      trackingDetails += htmlUtils.getOriginHtml(domain, action, show_breakage_warning);
+      out += htmlUtils.getOriginHtml(domain, action, show_breakage_warning);
     }
   }
 
   // Display tracking domains.
-  $('#blockedResourcesInner').html(trackingDetails);
+  $('#blockedResourcesInner').html(out);
 
   $('#blockedResourcesInner').off("scroll");
   $('#blockedResourcesInner').on("scroll", domains, addOrigins);
@@ -734,19 +708,14 @@ function showTrackingDomains(domains) {
   // activate tooltips
   $('#blockedResourcesInner .tooltip:not(.tooltipstered)').tooltipster(
     htmlUtils.DOMAIN_TOOLTIP_CONF);
-
-  // Register handlers for tracking domain toggle controls.
-  $('.switch-toggle').each(function() {
-    registerToggleHandlers($(this));
-  });
-
 }
+
 /**
  * https://tools.ietf.org/html/draft-ietf-rtcweb-ip-handling-01#page-5
  *
  * Toggle WebRTC IP address leak protection setting.
  *
- * When enabled, policy is set to Mode 4 (disable_non_proxied_udp).
+ * When enabled, policy is set to Mode 3 (default_public_interface_only).
  */
 function toggleWebRTCIPProtection() {
   // Return early with non-supporting browsers
@@ -758,44 +727,37 @@ function toggleWebRTCIPProtection() {
 
   cpn.webRTCIPHandlingPolicy.get({}, function (result) {
     // Update new value to be opposite of current browser setting
-    if (result.value == 'disable_non_proxied_udp') {
+    if (result.value == 'default_public_interface_only') {
       cpn.webRTCIPHandlingPolicy.clear({});
     } else {
       cpn.webRTCIPHandlingPolicy.set({
-        value: 'disable_non_proxied_udp'
+        value: 'default_public_interface_only'
       });
     }
   });
 }
 
 /**
- * Update the user preferences displayed for this origin.
- * These UI changes will later be used to update user preferences data.
- *
- * @param {Event} event Click event triggered by user.
+ * Updates domain tooltip, slider color.
+ * Also toggles status indicators like breakage warnings.
  */
-//TODO unduplicate this code? since it's also in popup
-function updateOrigin(event) {
-  // get the origin and new action for it
-  var $elm = $('label[for="' + event.currentTarget.id + '"]');
-  var action = $elm.data('action');
+function updateOrigin(origin, action, userset) {
+  let $clicker = $('#blockedResourcesInner div.clicker[data-origin="' + origin + '"]'),
+    $switchContainer = $clicker.find('.switch-container').first();
 
-  // replace the old action with the new one
-  var $switchContainer = $elm.parents('.switch-container').first();
+  // update slider color via CSS
   $switchContainer.removeClass([
     constants.BLOCK,
     constants.COOKIEBLOCK,
     constants.ALLOW,
     constants.NO_TRACKING].join(" ")).addClass(action);
 
-  let $clicker = $elm.parents('.clicker').first(),
-    origin = $clicker.data('origin'),
-    show_breakage_warning = (
-      action == constants.BLOCK &&
-      OPTIONS_DATA.cookieblocked.hasOwnProperty(origin)
-    );
+  let show_breakage_warning = (
+    action == constants.BLOCK &&
+    OPTIONS_DATA.cookieblocked.hasOwnProperty(origin)
+  );
 
-  htmlUtils.toggleBlockedStatus($clicker, action, show_breakage_warning);
+  htmlUtils.toggleBlockedStatus($clicker, userset, show_breakage_warning);
 
   // reinitialize the domain tooltip
   $clicker.find('.origin-inner').tooltipster('destroy');
@@ -804,26 +766,92 @@ function updateOrigin(event) {
   $clicker.find('.origin-inner').tooltipster(htmlUtils.DOMAIN_TOOLTIP_CONF);
 }
 
+/**
+ * Updates the list of tracking domains in response to user actions.
+ *
+ * For example, moving the slider for example.com should move the sliders
+ * for www.example.com and cdn.example.com
+ */
+function updateSliders(updatedOriginData) {
+  let updated_domains = Object.keys(updatedOriginData);
+
+  // update any sliders that changed
+  for (let domain of updated_domains) {
+    let action = updatedOriginData[domain];
+    if (action == OPTIONS_DATA.origins[domain]) {
+      continue;
+    }
+
+    let userset = false;
+    if (action.startsWith('user')) {
+      userset = true;
+      action = action.slice(5);
+    }
+
+    // update slider position
+    let $radios = $('#blockedResourcesInner div.clicker[data-origin="' + domain + '"] input'),
+      selected_val = (action == constants.DNT ? constants.ALLOW : action);
+    // update the radio group without triggering a change event
+    // https://stackoverflow.com/a/22635728
+    $radios.val([selected_val]);
+
+    // update domain slider row tooltip/status indicators
+    updateOrigin(domain, action, userset);
+  }
+
+  // remove sliders that are no longer present
+  let removed = Object.keys(OPTIONS_DATA.origins).filter(
+    x => !updated_domains.includes(x));
+  for (let domain of removed) {
+    let $clicker = $('#blockedResourcesInner div.clicker[data-origin="' + domain + '"]');
+    $clicker.remove();
+  }
+}
+
+/**
+ * Save the user setting for a domain by messaging the background page.
+ */
+function saveToggle(origin, action) {
+  chrome.runtime.sendMessage({
+    type: "saveOptionsToggle",
+    origin,
+    action
+  }, (response) => {
+    // first update the cache for the slider
+    // that was just changed by the user
+    // to avoid redundantly updating it below
+    OPTIONS_DATA.origins[origin] = response.origins[origin];
+    // update any sliders that changed as a result
+    updateSliders(response.origins);
+    // update cached domain data
+    OPTIONS_DATA.origins = response.origins;
+  });
+}
+
 /**
  * Remove origin from Privacy Badger.
  * @param {Event} event Click event triggered by user.
  */
 function removeOrigin(event) {
-  // Confirm removal before proceeding.
-  var removalConfirmed = confirm(i18n.getMessage("options_remove_origin_confirm"));
-  if (!removalConfirmed) {
+  event.preventDefault();
+
+  // confirm removal before proceeding
+  if (!confirm(i18n.getMessage("options_remove_origin_confirm"))) {
     return;
   }
 
-  // Remove traces of origin from storage.
-  var $element = $(event.target).parent();
-  var origin = $element.data('origin');
+  let origin = $(event.target).parent().data('origin');
+
   chrome.runtime.sendMessage({
     type: "removeOrigin",
-    origin: origin
+    origin
   }, (response) => {
+    // remove rows that are no longer here
+    updateSliders(response.origins);
+    // update cached domain data
     OPTIONS_DATA.origins = response.origins;
-    reloadTrackingDomainsTab();
+    // if we removed domains, the summary text may have changed
+    updateSummary();
   });
 }
 
@@ -840,6 +868,8 @@ function updateWidgetReplacementExceptions() {
 }
 
 $(function () {
+  $.tooltipster.setDefaults(htmlUtils.TOOLTIPSTER_DEFAULTS);
+
   chrome.runtime.sendMessage({
     type: "getOptionsData",
   }, (response) => {
