@@ -26,18 +26,6 @@ var htmlUtils = require("htmlutils").htmlUtils;
 
 let POPUP_DATA = {};
 
-// TODO hack: disable Tooltipster tooltips on Firefox
-// to avoid hangs on pages with enough domains to produce a scrollbar
-(function () {
-const matches = navigator.userAgent.match(
-  // from https://gist.github.com/ticky/3909462
-  /(MSIE|(?!Gecko.+)Firefox|(?!AppleWebKit.+Chrome.+)Safari|(?!AppleWebKit.+)Chrome|AppleWebKit(?!.+Chrome|.+Safari)|Gecko(?!.+Firefox))(?: |\/)([\d.apre]+)/
-);
-if (!matches || matches[1] == "Firefox") {
-  $.fn.tooltipster = function () {};
-}
-}());
-
 /* if they aint seen the comic*/
 function showNagMaybe() {
   var nag = $("#instruction");
@@ -60,8 +48,11 @@ function showNagMaybe() {
     nag.show();
     outer.show();
     // Attach event listeners
-    $('#fittslaw').on("click", _hideNag);
-    $("#firstRun").on("click", function() {
+    $('#fittslaw').on("click", function (e) {
+      e.preventDefault();
+      _hideNag();
+    });
+    $("#firstRun").on("click", function () {
       // If there is a firstRun.html tab, switch to the tab.
       // Otherwise, create a new tab
       chrome.tabs.query({url: firstRunUrl}, function (tabs) {
@@ -75,6 +66,7 @@ function showNagMaybe() {
           });
         }
         _hideNag();
+        window.close();
       });
     });
   }
@@ -88,7 +80,7 @@ function showNagMaybe() {
     });
   } else if (POPUP_DATA.criticalError) {
     $('#instruction-text').hide();
-    $('#error-text').show().find('a').attr('id', 'firstRun').css('padding', '5px');
+    $('#error-text').show().find('a').attr('id', 'critical-error-link').css('padding', '5px');
     $('#error-message').text(POPUP_DATA.criticalError);
     _showNag();
   }
@@ -127,16 +119,17 @@ function init() {
   $("#error").on("click", function() {
     overlay.toggleClass('active');
   });
-  $("#report_cancel").on("click", function() {
+  $("#report-cancel").on("click", function() {
     clearSavedErrorText();
     closeOverlay();
   });
-  $("#report_button").on("click", function() {
+  $("#report-button").on("click", function() {
     $(this).prop("disabled", true);
-    $("#report_cancel").prop("disabled", true);
+    $("#report-cancel").prop("disabled", true);
     send_error($("#error_input").val());
   });
-  $("#report_close").on("click", function() {
+  $("#report_close").on("click", function (e) {
+    e.preventDefault();
     clearSavedErrorText();
     closeOverlay();
   });
@@ -152,20 +145,26 @@ function init() {
     browser.runtime.getBrowserInfo().then(function (info) {
       if (info.name == "Firefox") {
         $("#options").on("click", function (e) {
-          openOptionsPage();
           e.preventDefault();
+          openPage(chrome.runtime.getURL("/skin/options.html"));
+        });
+        $("#help").on("click", function (e) {
+          e.preventDefault();
+          openPage(this.getAttribute('href'));
         });
       }
     });
   }
 
-  let shareOverlay = $("#share_overlay");
-
-  $("#share").on("click", share);
-  $("#share_close").on("click", function() {
-    shareOverlay.toggleClass('active', false);
+  $("#share").on("click", function (e) {
+    e.preventDefault();
+    share();
   });
-  $("#copy_button").on("click", function() {
+  $("#share_close").on("click", function (e) {
+    e.preventDefault();
+    $("#share_overlay").toggleClass('active', false);
+  });
+  $("#copy-button").on("click", function() {
     $("#share_output").select();
     document.execCommand('copy');
     $(this).text(chrome.i18n.getMessage("copy_button_copied"));
@@ -174,9 +173,7 @@ function init() {
   window.POPUP_INITIALIZED = true;
 }
 
-function openOptionsPage() {
-  const url = chrome.runtime.getURL("/skin/options.html");
-
+function openPage(url) {
   // first get the active tab
   chrome.tabs.query({ active: true, lastFocusedWindow: true }, (tabs) => {
     let activeTab = tabs[0],
@@ -196,6 +193,8 @@ function openOptionsPage() {
       delete tabProps.openerTabId;
       chrome.tabs.create(tabProps);
     }
+
+    window.close();
   });
 }
 
@@ -211,8 +210,8 @@ function clearSavedErrorText() {
  */
 function closeOverlay() {
   $('#overlay').toggleClass('active', false);
-  $("#report_success").toggleClass("hidden", true);
-  $("#report_fail").toggleClass("hidden", true);
+  $("#report-success").hide();
+  $("#report-fail").hide();
   $("#error_input").val("");
 }
 
@@ -250,7 +249,7 @@ function send_error(message) {
         action = "usernoaction";
       } else if (action == constants.USER_BLOCK) {
         action = "userblock";
-      } else if (action == constants.USER_COOKIE_BLOCK) {
+      } else if (action == constants.USER_COOKIEBLOCK) {
         action = "usercookieblock";
       } else if (action == constants.ALLOW) {
         action = "noaction";
@@ -276,25 +275,24 @@ function send_error(message) {
 
     sendReport.done(function() {
       $("#error_input").val("");
-      $("#report_success").toggleClass("hidden", false);
+      $("#report-success").slideDown();
 
       clearSavedErrorText();
 
       setTimeout(function() {
-        $("#report_button").prop("disabled", false);
-        $("#report_cancel").prop("disabled", false);
-        $("#report_success").toggleClass("hidden", true);
+        $("#report-button").prop("disabled", false);
+        $("#report-cancel").prop("disabled", false);
         closeOverlay();
       }, 3000);
     });
 
     sendReport.fail(function() {
-      $("#report_fail").toggleClass("hidden");
+      $("#report-fail").slideDown();
 
       setTimeout(function() {
-        $("#report_button").prop("disabled", false);
-        $("#report_cancel").prop("disabled", false);
-        $("#report_fail").toggleClass("hidden", true);
+        $("#report-button").prop("disabled", false);
+        $("#report-cancel").prop("disabled", false);
+        $("#report-fail").slideUp();
       }, 3000);
     });
   });
@@ -343,11 +341,12 @@ function deactivateOnSite() {
  */
 function share() {
   $("#share_overlay").toggleClass('active');
-  let shareMessage = chrome.i18n.getMessage("share_base_message");
+  let share_msg = chrome.i18n.getMessage("share_base_message");
 
-  //Only add language about found trackers if we actually found trackers (but regardless of whether we are actually blocking them).
+  // only add language about found trackers if we actually found trackers
+  // (but regardless of whether we are actually blocking them)
   if (POPUP_DATA.noTabData) {
-    $("#share_output").val(shareMessage);
+    $("#share_output").val(share_msg);
     return;
   }
 
@@ -358,15 +357,14 @@ function share() {
   }
 
   if (!originsArr.length) {
-    $("#share_output").val(shareMessage);
+    $("#share_output").val(share_msg);
     return;
   }
 
   originsArr = htmlUtils.sortDomains(originsArr);
   let tracking = [];
 
-  for (let i=0; i < originsArr.length; i++) {
-    let origin = originsArr[i];
+  for (let origin of originsArr) {
     let action = origins[origin];
 
     if (action != constants.NO_TRACKING) {
@@ -375,23 +373,23 @@ function share() {
   }
 
   if (tracking.length) {
-    shareMessage += "\n\n" + chrome.i18n.getMessage("share_tracker_header", [tracking.length, POPUP_DATA.tabHost]) + "\n\n";
-
-    for (let i=0; i < tracking.length; i++) {
-      shareMessage += tracking[i] + "\n";
-    }
+    share_msg += "\n\n";
+    share_msg += chrome.i18n.getMessage(
+      "share_tracker_header", [tracking.length, POPUP_DATA.tabHost]);
+    share_msg += "\n\n";
+    share_msg += tracking.join("\n");
   }
-  $("#share_output").val(shareMessage);
+  $("#share_output").val(share_msg);
 }
 
 /**
  * Handler to undo user selection for a tracker
- *
- * @param {Event} e The object the event triggered on
  */
-function revertDomainControl(e) {
-  var $elm = $(e.target).parent();
-  var origin = $elm.data('origin');
+function revertDomainControl(event) {
+  event.preventDefault();
+
+  let origin = $(event.target).parent().data('origin');
+
   chrome.runtime.sendMessage({
     type: "revertDomainControl",
     origin: origin
@@ -401,34 +399,6 @@ function revertDomainControl(e) {
   });
 }
 
-function registerToggleHandlers() {
-  // (this == .switch-toggle)
-  var radios = $(this).children('input');
-  var value = $(this).children('input:checked').val();
-  //var userHandle = $(this).children('a');
-
-  var slider = $("<div></div>").slider({
-    min: 0,
-    max: 2,
-    value: value,
-    create: function(/*event, ui*/) {
-      // Set the margin for the handle of the slider we're currently creating,
-      // depending on its blocked/cookieblocked/allowed value (this == .ui-slider)
-      $(this).children('.ui-slider-handle').css('margin-left', -16 * value + 'px');
-    },
-    slide: function(event, ui) {
-      radios.filter("[value=" + ui.value + "]").click();
-    },
-    stop: function(event, ui) {
-      $(ui.handle).css('margin-left', -16 * ui.value + "px");
-    },
-  }).appendTo(this);
-
-  radios.on("change", function () {
-    slider.slider("value", radios.filter(':checked').val());
-  });
-}
-
 /**
  * Refresh the content of the popup window
  *
@@ -507,8 +477,7 @@ function refreshPopup() {
   let nonTracking = [];
   originsArr = htmlUtils.sortDomains(originsArr);
 
-  for (let i=0; i < originsArr.length; i++) {
-    let origin = originsArr[i];
+  for (let origin of originsArr) {
     let action = origins[origin];
 
     if (action == constants.NO_TRACKING) {
@@ -585,7 +554,7 @@ function refreshPopup() {
     $('#instructions-many-trackers').html(chrome.i18n.getMessage(
       "popup_instructions", [
         POPUP_DATA.trackerCount,
-        "<a target='_blank' title='" + _.escape(chrome.i18n.getMessage("what_is_a_tracker")) + "' class='tooltip' href='https://www.eff.org/privacybadger/faq#What-is-a-third-party-tracker'>"
+        "<a target='_blank' title='" + _.escape(chrome.i18n.getMessage("what_is_a_tracker")) + "' class='tooltip' href='https://privacybadger.org/#What-is-a-third-party-tracker'>"
       ]
     )).find(".tooltip").tooltipster();
   }
@@ -595,8 +564,6 @@ function refreshPopup() {
 
     let $printable = $(printable.splice(0, CHUNK).join(""));
 
-    $printable.find('.switch-toggle').each(registerToggleHandlers);
-
     // Hide elements for removing origins (controlled from the options page).
     // Popup shows what's loaded for the current page so it doesn't make sense
     // to have removal ability here.
@@ -628,27 +595,27 @@ function refreshPopup() {
  *
  * @param {Event} event Click event triggered by user.
  */
-function updateOrigin(event) {
+function updateOrigin() {
   // get the origin and new action for it
-  var $elm = $('label[for="' + event.currentTarget.id + '"]');
-  var action = $elm.data('action');
+  let $radio = $(this),
+    action = $radio.val(),
+    $switchContainer = $radio.parents('.switch-container').first();
 
-  // replace the old action with the new one
-  var $switchContainer = $elm.parents('.switch-container').first();
+  // update slider color via CSS
   $switchContainer.removeClass([
     constants.BLOCK,
     constants.COOKIEBLOCK,
     constants.ALLOW,
     constants.NO_TRACKING].join(" ")).addClass(action);
 
-  let $clicker = $elm.parents('.clicker').first(),
+  let $clicker = $radio.parents('.clicker').first(),
     origin = $clicker.data('origin'),
     show_breakage_warning = (
       action == constants.BLOCK &&
       POPUP_DATA.cookieblocked.hasOwnProperty(origin)
     );
 
-  htmlUtils.toggleBlockedStatus($clicker, action, show_breakage_warning);
+  htmlUtils.toggleBlockedStatus($clicker, true, show_breakage_warning);
 
   // reinitialize the domain tooltip
   $clicker.find('.origin-inner').tooltipster('destroy');
@@ -657,32 +624,19 @@ function updateOrigin(event) {
   $clicker.find('.origin-inner').tooltipster(htmlUtils.DOMAIN_TOOLTIP_CONF);
 
   // persist the change
-  saveToggle($clicker);
+  saveToggle(origin, action);
 }
 
 /**
  * Save the user setting for a domain by messaging the background page.
  */
-function saveToggle($clicker) {
-  let origin = $clicker.attr("data-origin"),
-    action;
-
-  if ($clicker.hasClass(constants.BLOCK)) {
-    action = constants.BLOCK;
-  } else if ($clicker.hasClass(constants.COOKIEBLOCK)) {
-    action = constants.COOKIEBLOCK;
-  } else if ($clicker.hasClass(constants.ALLOW)) {
-    action = constants.ALLOW;
-  }
-
-  if (action) {
-    chrome.runtime.sendMessage({
-      type: "savePopupToggle",
-      origin: origin,
-      action: action,
-      tabId: POPUP_DATA.tabId
-    });
-  }
+function saveToggle(origin, action) {
+  chrome.runtime.sendMessage({
+    type: "savePopupToggle",
+    origin,
+    action,
+    tabId: POPUP_DATA.tabId
+  });
 }
 
 function getTab(callback) {
@@ -703,6 +657,8 @@ function setPopupData(data) {
 }
 
 $(function () {
+  $.tooltipster.setDefaults(htmlUtils.TOOLTIPSTER_DEFAULTS);
+
   getTab(function (tab) {
     chrome.runtime.sendMessage({
       type: "getPopupData",
