@@ -24,17 +24,17 @@
 
 /* globals badger:false, log:false */
 
-require.scopes.webrequest = (function() {
+require.scopes.webrequest = (function () {
 
 /*********************** webrequest scope **/
 
-var constants = require("constants");
-var getSurrogateURI = require("surrogates").getSurrogateURI;
-var incognito = require("incognito");
-var utils = require("utils");
+let constants = require("constants"),
+  getSurrogateURI = require("surrogates").getSurrogateURI,
+  incognito = require("incognito"),
+  utils = require("utils");
 
 /************ Local Variables *****************/
-var temporaryWidgetUnblock = {};
+let tempAllowList = {};
 
 /***************** Blocking Listener Functions **************/
 
@@ -45,7 +45,7 @@ var temporaryWidgetUnblock = {};
  * @returns {Object} Can cancel requests
  */
 function onBeforeRequest(details) {
-  var frame_id = details.frameId,
+  let frame_id = details.frameId,
     tab_id = details.tabId,
     type = details.type,
     url = details.url;
@@ -78,41 +78,40 @@ function onBeforeRequest(details) {
     return {};
   }
 
-  var requestAction = checkAction(tab_id, request_host, frame_id);
-  if (!requestAction) {
+  let action = checkAction(tab_id, request_host, frame_id);
+  if (!action) {
     return {};
   }
 
   // log the third-party domain asynchronously
   // (don't block a critical code path on updating the badge)
   setTimeout(function () {
-    badger.logThirdPartyOriginOnTab(tab_id, request_host, requestAction);
+    badger.logThirdPartyOriginOnTab(tab_id, request_host, action);
   }, 0);
 
   if (!badger.isPrivacyBadgerEnabled(tab_host)) {
     return {};
   }
 
-  if (requestAction != constants.BLOCK && requestAction != constants.USER_BLOCK) {
+  if (action != constants.BLOCK && action != constants.USER_BLOCK) {
     return {};
   }
 
   if (type == 'script') {
-    var surrogate = getSurrogateURI(url, request_host);
+    let surrogate = getSurrogateURI(url, request_host);
     if (surrogate) {
       return {redirectUrl: surrogate};
     }
   }
 
-  // Notify the content script...
-  var msg = {
+  // notify the widget replacement content script
+  chrome.tabs.sendMessage(tab_id, {
     replaceWidget: true,
     trackerDomain: request_host
-  };
-  chrome.tabs.sendMessage(tab_id, msg);
+  });
 
   // if this is a heuristically- (not user-) blocked domain
-  if (requestAction == constants.BLOCK && incognito.learningEnabled(tab_id)) {
+  if (action == constants.BLOCK && incognito.learningEnabled(tab_id)) {
     // check for DNT policy asynchronously
     setTimeout(function () {
       badger.checkForDNTPolicy(request_host);
@@ -175,12 +174,12 @@ function onBeforeSendHeaders(details) {
     }
   }
 
-  var requestAction = checkAction(tab_id, request_host, frame_id);
+  let action = checkAction(tab_id, request_host, frame_id);
 
-  if (requestAction) {
+  if (action) {
     // log the third-party domain asynchronously
     setTimeout(function () {
-      badger.logThirdPartyOriginOnTab(tab_id, request_host, requestAction);
+      badger.logThirdPartyOriginOnTab(tab_id, request_host, action);
     }, 0);
   }
 
@@ -189,7 +188,7 @@ function onBeforeSendHeaders(details) {
   }
 
   // handle cookieblocked requests
-  if (requestAction == constants.COOKIEBLOCK || requestAction == constants.USER_COOKIE_BLOCK) {
+  if (action == constants.COOKIEBLOCK || action == constants.USER_COOKIEBLOCK) {
     let newHeaders;
 
     // GET requests: remove cookie headers, reduce referrer header to origin
@@ -236,7 +235,7 @@ function onBeforeSendHeaders(details) {
  * @returns {Object} The new response headers
  */
 function onHeadersReceived(details) {
-  var tab_id = details.tabId,
+  let tab_id = details.tabId,
     url = details.url;
 
   if (_isTabChromeInternal(tab_id)) {
@@ -266,28 +265,28 @@ function onHeadersReceived(details) {
   }
 
   let tab_host = getHostForTab(tab_id);
-  let request_host = window.extractHostFromURL(url);
+  let response_host = window.extractHostFromURL(url);
 
-  if (!utils.isThirdPartyDomain(request_host, tab_host)) {
+  if (!utils.isThirdPartyDomain(response_host, tab_host)) {
     return {};
   }
 
-  var requestAction = checkAction(tab_id, request_host, details.frameId);
-  if (!requestAction) {
+  let action = checkAction(tab_id, response_host, details.frameId);
+  if (!action) {
     return {};
   }
 
   // log the third-party domain asynchronously
   setTimeout(function () {
-    badger.logThirdPartyOriginOnTab(tab_id, request_host, requestAction);
+    badger.logThirdPartyOriginOnTab(tab_id, response_host, action);
   }, 0);
 
   if (!badger.isPrivacyBadgerEnabled(tab_host)) {
     return {};
   }
 
-  if (requestAction == constants.COOKIEBLOCK || requestAction == constants.USER_COOKIE_BLOCK) {
-    var newHeaders = details.responseHeaders.filter(function(header) {
+  if (action == constants.COOKIEBLOCK || action == constants.USER_COOKIEBLOCK) {
+    let newHeaders = details.responseHeaders.filter(function(header) {
       return (header.name.toLowerCase() != "set-cookie");
     });
     return {responseHeaders: newHeaders};
@@ -307,6 +306,7 @@ function onTabRemoved(tabId) {
 
 /**
  * Update internal db on tabs when a tab gets replaced
+ * due to prerendering or instant search.
  *
  * @param {Integer} addedTabId The new tab id that replaces
  * @param {Integer} removedTabId The tab id that gets removed
@@ -359,7 +359,7 @@ function onNavigate(details) {
  * @return {String} the host name for the tab
  */
 function getHostForTab(tabId) {
-  var mainFrameIdx = 0;
+  let mainFrameIdx = 0;
   if (!badger.tabData[tabId]) {
     return '';
   }
@@ -397,7 +397,10 @@ function recordSuperCookie(tab_id, frame_url) {
   }
 
   badger.heuristicBlocking.updateTrackerPrevalence(
-    frame_host, window.getBaseDomain(page_host));
+    frame_host,
+    window.getBaseDomain(frame_host),
+    window.getBaseDomain(page_host)
+  );
 }
 
 /**
@@ -417,17 +420,17 @@ function recordFingerprinting(tabId, msg) {
   }
 
   // Ignore first-party scripts
-  var script_host = window.extractHostFromURL(msg.scriptUrl),
+  let script_host = window.extractHostFromURL(msg.scriptUrl),
     document_host = badger.getFrameData(tabId).host;
   if (!utils.isThirdPartyDomain(script_host, document_host)) {
     return;
   }
 
-  var CANVAS_WRITE = {
+  let CANVAS_WRITE = {
     fillText: true,
     strokeText: true
   };
-  var CANVAS_READ = {
+  let CANVAS_READ = {
     getImageData: true,
     toDataURL: true
   };
@@ -436,7 +439,7 @@ function recordFingerprinting(tabId, msg) {
     badger.tabData[tabId].fpData = {};
   }
 
-  var script_origin = window.getBaseDomain(script_host);
+  let script_origin = window.getBaseDomain(script_host);
 
   // Initialize script TLD-level data
   if (!badger.tabData[tabId].fpData.hasOwnProperty(script_origin)) {
@@ -447,7 +450,7 @@ function recordFingerprinting(tabId, msg) {
       }
     };
   }
-  var scriptData = badger.tabData[tabId].fpData[script_origin];
+  let scriptData = badger.tabData[tabId].fpData[script_origin];
 
   if (msg.extra.hasOwnProperty('canvas')) {
     if (scriptData.canvas.fingerprinting) {
@@ -466,7 +469,7 @@ function recordFingerprinting(tabId, msg) {
 
           // Mark this as a strike
           badger.heuristicBlocking.updateTrackerPrevalence(
-            script_host, window.getBaseDomain(document_host));
+            script_host, script_origin, window.getBaseDomain(document_host));
         }
       }
       // This is a canvas write
@@ -477,13 +480,13 @@ function recordFingerprinting(tabId, msg) {
 }
 
 /**
- * Delete tab data, de-register tab
+ * Cleans up tab-specific data.
  *
- * @param {Integer} tabId The id of the tab
+ * @param {Integer} tab_id the ID of the tab
  */
-function forgetTab(tabId) {
-  delete badger.tabData[tabId];
-  delete temporaryWidgetUnblock[tabId];
+function forgetTab(tab_id) {
+  delete badger.tabData[tab_id];
+  delete tempAllowList[tab_id];
 }
 
 /**
@@ -497,7 +500,7 @@ function forgetTab(tabId) {
 function checkAction(tabId, requestHost, frameId) {
   // Ignore requests from temporarily unblocked widgets.
   // Someone clicked the widget, so let it load.
-  if (isWidgetTemporaryUnblock(tabId, requestHost, frameId)) {
+  if (allowedOnTab(tabId, requestHost, frameId)) {
     return false;
   }
 
@@ -547,93 +550,164 @@ function _isTabAnExtension(tabId) {
 /**
  * Provides the widget replacing content script with list of widgets to replace.
  *
+ * @param {Integer} tab_id the ID of the tab we're replacing widgets in
+ *
  * @returns {Object} dict containing the complete list of widgets
  * as well as a mapping to indicate which ones should be replaced
  */
-let getWidgetBlockList = (function () {
+let getWidgetList = (function () {
   // cached translations
-  let translations = [];
+  let translations;
+
   // inputs to chrome.i18n.getMessage()
   const widgetTranslations = [
     {
       key: "social_tooltip_pb_has_replaced",
       placeholders: ["XXX"]
     },
+    {
+      key: "widget_placeholder_pb_has_replaced",
+      placeholders: ["XXX"]
+    },
     { key: "allow_once" },
   ];
 
-  return function () {
-    // A mapping of individual SocialWidget objects to boolean values that determine
-    // whether the content script should replace that tracker's button/widget
-    var widgetsToReplace = {};
+  return function (tab_id) {
+    // an object with keys set to widget names that should be replaced
+    let widgetsToReplace = {},
+      widgetList = [],
+      tabData = badger.tabData[tab_id],
+      tabOrigins = tabData && tabData.origins && Object.keys(tabData.origins),
+      exceptions = badger.getSettings().getItem('widgetReplacementExceptions');
 
-    // optimize translation lookups by doing them just once
+    // optimize translation lookups by doing them just once,
     // the first time they are needed
-    if (!translations.length) {
+    if (!translations) {
       translations = widgetTranslations.reduce((memo, data) => {
         memo[data.key] = chrome.i18n.getMessage(data.key, data.placeholders);
         return memo;
       }, {});
+
+      // TODO duplicated in src/lib/i18n.js
+      const RTL_LOCALES = ['ar', 'he', 'fa'],
+        UI_LOCALE = chrome.i18n.getMessage('@@ui_locale');
+      translations.rtl = RTL_LOCALES.indexOf(UI_LOCALE) > -1;
     }
 
-    badger.widgetList.forEach(function (widget) {
-      // replace blocked widgets only
-      // and only if the widget is not on the 'do not replace' list
-      const replace = !badger.getSettings().getItem('widgetReplacementExceptions').includes(widget.name);
-      const action = badger.storage.getBestAction(widget.domain);
+    for (let widget of badger.widgetList) {
+      // replace only if the widget is not on the 'do not replace' list
+      // also don't send widget data used later for dynamic replacement
+      if (exceptions.includes(widget.name)) {
+        continue;
+      }
 
-      widgetsToReplace[widget.name] = replace && (
-        action == constants.BLOCK ||
-        action == constants.USER_BLOCK
-      );
-    });
+      widgetList.push(widget);
+
+      // replace only if at least one of the associated domains was blocked
+      if (!tabOrigins || !tabOrigins.length) {
+        continue;
+      }
+      let replace = widget.domains.some(domain => {
+        // leading wildcard domain
+        if (domain[0] == "*") {
+          domain = domain.slice(1);
+          // get all domains in tabData.origins that end with this domain
+          let matches = tabOrigins.filter(origin => {
+            return origin.endsWith(domain);
+          });
+          // do we have any matches and are they all blocked?
+          return matches.length && matches.every(origin => {
+            const action = tabData.origins[origin];
+            return (
+              action == constants.BLOCK ||
+              action == constants.USER_BLOCK
+            );
+          });
+        }
+
+        // regular, non-leading wildcard domain
+        if (!tabData.origins.hasOwnProperty(domain)) {
+          return false;
+        }
+        const action = tabData.origins[domain];
+        return (
+          action == constants.BLOCK ||
+          action == constants.USER_BLOCK
+        );
+
+      });
+      if (replace) {
+        widgetsToReplace[widget.name] = true;
+      }
+    }
 
     return {
       translations,
-      trackers: badger.widgetList,
-      trackerButtonsToReplace: widgetsToReplace
+      widgetList,
+      widgetsToReplace
     };
   };
 }());
 
 /**
- * Check if tab is temporarily unblocked for tracker
+ * Checks if given FQDN is temporarily unblocked on a tab.
  *
- * @param {Integer} tabId id of the tab to check
- * @param {String} requestHost FQDN to check
- * @param {Integer} frameId frame id to check
- * @returns {Boolean} true if in exception list
+ * @param {Integer} tab_id the ID of the tab to check
+ * @param {String} request_host the request FQDN to check
+ * @param {Integer} frame_id the frame ID to check
+ *
+ * @returns {Boolean} true if FQDN is on the temporary allow list
  */
-function isWidgetTemporaryUnblock(tabId, requestHost, frameId) {
-  var exceptions = temporaryWidgetUnblock[tabId];
-  if (exceptions === undefined) {
+function allowedOnTab(tab_id, request_host, frame_id) {
+  if (!tempAllowList.hasOwnProperty(tab_id)) {
     return false;
   }
 
-  var requestExcept = (exceptions.indexOf(requestHost) != -1);
+  let exceptions = tempAllowList[tab_id];
 
-  var frameHost = badger.getFrameData(tabId, frameId).host;
-  var frameExcept = (exceptions.indexOf(frameHost) != -1);
+  for (let exception of exceptions) {
+    if (exception == request_host) {
+      return true;
+    // leading wildcard
+    } else if (exception[0] == "*") {
+      if (request_host.endsWith(exception.slice(1))) {
+        return true;
+      }
+    }
+  }
 
-  return (requestExcept || frameExcept);
+  let frameData = badger.getFrameData(tab_id, frame_id);
+  if (!frameData || !frameData.host) {
+    return false;
+  }
+
+  let frame_host = frameData.host;
+  for (let exception of exceptions) {
+    if (exception == frame_host) {
+      return true;
+    // leading wildcard
+    } else if (exception[0] == "*") {
+      if (frame_host.endsWith(exception.slice(1))) {
+        return true;
+      }
+    }
+  }
+
+  return false;
 }
 
 /**
- * Unblocks a tracker just temporarily on this tab, because the user has clicked the
- * corresponding replacement widget.
+ * Marks a set of (widget) domains to be (temporarily) allowed on a tab.
  *
- * @param {Integer} tabId The id of the tab
- * @param {Array} widgetUrls an array of widget urls
+ * @param {Integer} tab_id the ID of the tab
+ * @param {Array} domains the domains
  */
-function unblockWidgetOnTab(tabId, widgetUrls) {
-  if (temporaryWidgetUnblock[tabId] === undefined) {
-    temporaryWidgetUnblock[tabId] = [];
+function allowOnTab(tab_id, domains) {
+  if (!tempAllowList.hasOwnProperty(tab_id)) {
+    tempAllowList[tab_id] = [];
   }
-  for (let i in widgetUrls) {
-    let url = widgetUrls[i];
-    // TODO just store actual domains in the JSON in the first place
-    let host = window.extractHostFromURL(url);
-    temporaryWidgetUnblock[tabId].push(host);
+  for (let domain of domains) {
+    tempAllowList[tab_id].push(domain);
   }
 }
 
@@ -649,7 +723,6 @@ function dispatcher(request, sender, sendResponse) {
       "checkEnabled",
       "checkEnabledAndThirdParty",
       "checkLocation",
-      "checkReplaceButton",
       "checkWidgetReplacementEnabled",
       "fpReport",
       "getReplacementButton",
@@ -691,16 +764,7 @@ function dispatcher(request, sender, sendResponse) {
     }
 
     let action = checkAction(sender.tab.id, frame_host);
-    sendResponse(action == constants.COOKIEBLOCK || action == constants.USER_COOKIE_BLOCK);
-
-    break;
-  }
-
-  case "checkReplaceButton": {
-    if (badger.isPrivacyBadgerEnabled(window.extractHostFromURL(sender.tab.url)) && badger.isWidgetReplacementEnabled()) {
-      let widgetBlockList = getWidgetBlockList();
-      sendResponse(widgetBlockList);
-    }
+    sendResponse(action == constants.COOKIEBLOCK || action == constants.USER_COOKIEBLOCK);
 
     break;
   }
@@ -713,7 +777,7 @@ function dispatcher(request, sender, sendResponse) {
         !widgetData.replacementButton.unblockDomains) {
       return sendResponse();
     }
-    unblockWidgetOnTab(sender.tab.id, widgetData.replacementButton.unblockDomains);
+    allowOnTab(sender.tab.id, widgetData.replacementButton.unblockDomains);
     sendResponse();
     break;
   }
@@ -789,10 +853,16 @@ function dispatcher(request, sender, sendResponse) {
   }
 
   case "checkWidgetReplacementEnabled": {
-    sendResponse(
-      badger.isPrivacyBadgerEnabled(window.extractHostFromURL(sender.tab.url)) &&
-      badger.isWidgetReplacementEnabled()
-    );
+    let response = false,
+      tab_host = window.extractHostFromURL(sender.tab.url);
+
+    if (badger.isPrivacyBadgerEnabled(tab_host) &&
+        badger.isWidgetReplacementEnabled()) {
+      response = getWidgetList(sender.tab.id);
+    }
+
+    sendResponse(response);
+
     break;
   }
 
@@ -814,7 +884,7 @@ function dispatcher(request, sender, sendResponse) {
 
     for (let origin in origins) {
       // see if origin would be cookieblocked if not for user override
-      if (badger.storage.getCookieblockStatus(origin)) {
+      if (badger.storage.wouldGetCookieblocked(origin)) {
         cookieblocked[origin] = true;
       }
     }
@@ -843,7 +913,7 @@ function dispatcher(request, sender, sendResponse) {
     let cookieblocked = {};
     for (let origin in origins) {
       // see if origin would be cookieblocked if not for user override
-      if (badger.storage.getCookieblockStatus(origin)) {
+      if (badger.storage.wouldGetCookieblocked(origin)) {
         cookieblocked[origin] = true;
       }
     }
@@ -869,9 +939,14 @@ function dispatcher(request, sender, sendResponse) {
 
   case "resetData": {
     badger.storage.clearTrackerData();
-    badger.loadSeedData();
-    sendResponse();
-    break;
+    badger.loadSeedData(err => {
+      if (err) {
+        console.error(err);
+      }
+      sendResponse();
+    });
+    // indicate this is an async response to chrome.runtime.onMessage
+    return true;
   }
 
   case "removeAllData": {
@@ -887,14 +962,14 @@ function dispatcher(request, sender, sendResponse) {
 
   case "activateOnSite": {
     badger.enablePrivacyBadgerForOrigin(request.tabHost);
-    badger.refreshIconAndContextMenu(request.tabId, request.tabUrl);
+    badger.updateIcon(request.tabId, request.tabUrl);
     sendResponse();
     break;
   }
 
   case "deactivateOnSite": {
     badger.disablePrivacyBadgerForOrigin(request.tabHost);
-    badger.refreshIconAndContextMenu(request.tabId, request.tabUrl);
+    badger.updateIcon(request.tabId, request.tabUrl);
     sendResponse();
     break;
   }
@@ -912,14 +987,14 @@ function dispatcher(request, sender, sendResponse) {
       if (chrome.runtime.lastError) {
         sendResponse({success: false, message: chrome.runtime.lastError.message});
       } else if (store.hasOwnProperty("disabledSites")) {
-        let whitelist = _.union(
+        let disabledSites = _.union(
           badger.getDisabledSites(),
           store.disabledSites
         );
-        badger.getSettings().setItem("disabledSites", whitelist);
+        badger.getSettings().setItem("disabledSites", disabledSites);
         sendResponse({
           success: true,
-          disabledSites: whitelist
+          disabledSites
         });
       } else {
         sendResponse({
@@ -1075,8 +1150,9 @@ function startListeners() {
 }
 
 /************************************** exports */
-var exports = {};
-exports.startListeners = startListeners;
+let exports = {
+  startListeners
+};
 return exports;
 /************************************** exports */
 })();
